/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.storage.database.pool;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;



public class ConnectionInfo {
    private static int nextId = 0;
    private int id = nextId++;

    private Connection connection;
    private Map<String, PreparedStatement> preparedStatementMap = new HashMap<String, PreparedStatement>();
    private Statement genericStatement;
    
    private long lastAccessTime;
    private boolean InformedUserOfLongConnectionTime = false;
    private Exception connectionCallStack;
    
    public ConnectionInfo(Connection connection) {
        this.connection = connection;
    }
    
    public int getId() {
        return id;
    }
    
    
    /**
     * Synchronized by the connection pool.
     */
    public void touch() {
        lastAccessTime = System.currentTimeMillis();
    }
    
    /**
     * Synchronized by the connection pool.
     */
    public long getLastAccessTime() {
        return lastAccessTime;
    }
    
    public boolean getInformedUserOfLongConnectionTime() {
        return InformedUserOfLongConnectionTime;
    }
    
    public void setInformedUserOfLongConnectionTime(boolean informedUserOfLongConnectionTime) {
        InformedUserOfLongConnectionTime = informedUserOfLongConnectionTime;
    }
    
    public Exception getConnectionCallStack() {
        return connectionCallStack;
    }
    
    public void setConnectionCallStack(Exception connectionCallStack) {
        this.connectionCallStack = connectionCallStack;
    }
    
    public Connection getConnection() {
        return connection;
    }
    
    public PreparedStatement getPreparedStatement(String text) throws SQLException {
        PreparedStatement stmt = (PreparedStatement) preparedStatementMap.get(text);
        if (stmt == null) {
            stmt = connection.prepareStatement(text);
            preparedStatementMap.put(text, stmt);
        }
        return stmt;
    }
    
    public synchronized Statement getStatement() throws SQLException {
        if (genericStatement == null) {
            genericStatement = connection.createStatement();
        }
        return genericStatement;
    }
    
    public void close() throws SQLException {
        closeStatements();
        connection.close();
    }
    
    public synchronized void closeStatements() throws SQLException {
        for (PreparedStatement stmt : preparedStatementMap.values()) {
            stmt.close();
        }
        preparedStatementMap.clear();
        if (genericStatement != null) {
            genericStatement.close();
            genericStatement = null;
        }
    }
    

}
